@@ -411,9 +411,107 @@ module REXMLTests
 
       s = "<a><b><c id='1'/></b><b><b><c id='2'/><c id='3'/></b><c id='4'/></b><c id='NOMATCH'><c id='5'/></c></a>"
       d = REXML::Document.new(s)
-      c = REXML::XPath.match( d, "//c[@id = '5']")
-      cs = REXML::XPath.match( c, "preceding::c" )
-      assert_equal( 4, cs.length )
+      c = REXML::XPath.match(d, "//c[@id = '5']")
+      assert_equal(1, c.length)
+      cs = REXML::XPath.match(c.first, "preceding::c")
+      assert_equal(4, cs.length)
+    end
+
+    def test_preceding_multiple
+      source = <<-XML
+<a>
+  <b/><c/><d/><d/><e/><f/>
+</a>
+      XML
+      doc = REXML::Document.new(source)
+      matches = REXML::XPath.match(doc, "a/d/preceding::*")
+      assert_equal(["d", "c", "b"], matches.map(&:name))
+    end
+
+    def test_following_multiple
+      source = <<-XML
+<a>
+  <b/><c/><d/><d/><e/><f/>
+</a>
+      XML
+      doc = REXML::Document.new(source)
+      matches = REXML::XPath.match(doc, "a/d/following::*")
+      assert_equal(["d", "e", "f"], matches.map(&:name))
+    end
+
+    def test_following_sibling_across_multiple_nodes
+      source = <<-XML
+<a>
+  <b>
+    <x/><c/><d/>
+  </b>
+  <b>
+    <x/><e/>
+  </b>
+</a>
+      XML
+      doc = REXML::Document.new(source)
+      matches = REXML::XPath.match(doc, "a/b/x/following-sibling::*")
+      assert_equal(["c", "d", "e"], matches.map(&:name))
+    end
+
+    def test_following_sibling_within_single_node
+      source = <<-XML
+<a>
+  <b>
+    <x/><c/><d/><x/><e/>
+  </b>
+</a>
+      XML
+      doc = REXML::Document.new(source)
+      matches = REXML::XPath.match(doc, "a/b/x/following-sibling::*")
+      assert_equal(["c", "d", "x", "e"], matches.map(&:name))
+    end
+
+    def test_following_sibling_predicates
+      source = <<-XML
+<div>
+  <div>
+    <a/><w/>
+  </div>
+  <a/><x/>
+  <a/><y/>
+  <a/><z/>
+</div>
+      XML
+      doc = REXML::Document.new(source)
+      # Finds a node flowing <a/>
+      matches = REXML::XPath.match(doc, "//a/following-sibling::*[1]")
+      assert_equal(["w", "x", "y", "z"], matches.map(&:name))
+    end
+
+    def test_preceding_sibling_across_multiple_nodes
+      source = <<-XML
+<a>
+  <b>
+    <c/><d/><x/>
+  </b>
+  <b>
+    <e/><x/>
+  </b>
+</a>
+      XML
+      doc = REXML::Document.new(source)
+      matches = REXML::XPath.match(doc, "a/b/x/preceding-sibling::*")
+      assert_equal(["e", "d", "c"], matches.map(&:name))
+    end
+
+    def test_preceding_sibling_within_single_node
+      source = <<-XML
+<a>
+  <b>
+    <c/><d/><x/><e/><x/>
+  </b>
+</a>
+      XML
+      doc = REXML::Document.new(source)
+      matches = REXML::XPath.match(doc, "a/b/x/preceding-sibling::*")
+      assert_equal(["e", "x", "d", "c"], matches.map(&:name))
     end
 
     def test_following
@@ -1095,6 +1193,16 @@ EOF
       assert_equal( 1,  XPath.match( d, "//x:*" ).size )
     end
 
+    def test_namespaces_cache
+      doc = Document.new("<a xmlns='1'><b/></a>")
+      assert_equal("<b/>", XPath.first(doc, "//b[namespace-uri()='1']").to_s)
+      assert_nil(XPath.first(doc, "//b[namespace-uri()='']"))
+
+      doc.root.delete_namespace
+      assert_nil(XPath.first(doc, "//b[namespace-uri()='1']"))
+      assert_equal("<b/>", XPath.first(doc, "//b[namespace-uri()='']").to_s)
+    end
+
     def test_ticket_71
       doc = Document.new(%Q{<root xmlns:ns1="xyz" xmlns:ns2="123"><element ns1:attrname="foo" ns2:attrname="bar"/></root>})
       el = doc.root.elements[1]
@@ -1158,5 +1266,15 @@ EOF
       end
       assert_equal(["/"], hrefs, "Bug #3842 [ruby-core:32447]")
     end
+
+    def test_match_with_deprecated_usage
+      verbose, $VERBOSE = $VERBOSE, nil
+      doc = Document.new("<a><b/></a>")
+      assert_equal(['b'], XPath.match([doc, doc], '//b').map(&:name))
+      assert_equal(['b'], XPath.match([doc], '//b').map(&:name))
+      assert_equal([], XPath.match([], '//b').map(&:name))
+    ensure
+      $VERBOSE = verbose
+    end
   end
 end
